//@flow

import {assert} from './helpers'

export const cellStates = {
    closed: 'closed',
    open: 'open',
    marked: 'marked',
    // legal transitions:
    //  closed --> marked (doMark)
    //  marked --> closed (doMark)
    //  closed --> open (doOpen)
}

export type CellState = $Keys<typeof cellStates>
export type CellConfig = {
  state: CellState,
  mined: bool,
  x: number,
  y: number,
  bombsAround?: number
}

export class Cell {
    x: number
    y: number
    bombsAround: number
    state: CellState
    mined: bool

    constructor({state, mined, x, y, bombsAround = 0}: CellConfig) {
        this.state = state
        this.mined = mined
        this.x = x
        this.y = y
        this.bombsAround = bombsAround
    }

    config(): CellConfig {
      const {state, mined, x, y, bombsAround} = this
      return {state, mined, x, y, bombsAround}
    }

    doMark() {
        if (this.state === cellStates.closed) {
            // do mark
            this.state = cellStates.marked
        }
        else if (this.state === cellStates.marked) {
            // do unmark
            this.state = cellStates.closed
        }
        else {
            throw new Error('doMark: Invalid state: ' + this.state)
        }
    }
    doOpen() {
        assert(this.state === cellStates.closed, 'doOpen: Invalid state: ' + this.state)
        this.state = cellStates.open
    }
}
